"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const ncp_1 = require("ncp");
const mkp = require("mkdirp");
const util_1 = require("util");
const path_1 = require("path");
const fs_1 = require("fs");
const semver = require("semver");
const extract = require("extract-zip");
const rmrf = require("rimraf");
const follow_redirects_1 = require("follow-redirects");
const pkg = require('../package.json');
const rimraf = util_1.promisify(rmrf);
const mkdirp = util_1.promisify(mkp);
const copy = util_1.promisify(ncp_1.ncp);
exports.getElectronVersion = (baseDir) => __awaiter(this, void 0, void 0, function* () {
    const pkg = JSON.parse(yield fs_1.promises.readFile(path_1.join(baseDir, 'package.json'), 'utf8'));
    let electronVersion;
    if (pkg) {
        if (pkg.devDependencies && pkg.devDependencies.electron) {
            electronVersion = semver.minVersion(pkg.devDependencies.electron);
        }
        else {
            throw new Error('Electron should be installed as a dev dependency.');
        }
    }
    else {
        throw new Error('package.json file does not exist in the specified directory.');
    }
    if (!electronVersion)
        throw new Error('Failed to parse Electron version.');
    return electronVersion;
});
exports.downloadBinaries = (os) => {
    return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
        const distPath = path_1.join(__dirname, `../download/${os}`);
        const versionPath = path_1.join(distPath, 'version');
        if (fs_1.existsSync(versionPath) &&
            (yield fs_1.promises.readFile(versionPath, 'utf8')) === pkg.version) {
            return resolve();
        }
        console.log('Downloading electron-global binaries');
        yield rimraf(distPath);
        yield mkdirp(distPath);
        const url = `https://github.com/sentialx/electron-global/releases/download/v${pkg.version}/electron-v${pkg.version}-${os}-ia32.zip`;
        const zipPath = path_1.join(distPath, `electron-v${pkg.version}-${os}-ia32.zip`);
        const stream = fs_1.createWriteStream(zipPath);
        const req = follow_redirects_1.https.get(url, res => {
            res.pipe(stream);
            res.on('end', () => {
                stream.end();
                extract(zipPath, { dir: path_1.join(__dirname, `../download/${os}`) }, (err) => __awaiter(this, void 0, void 0, function* () {
                    if (err)
                        return reject(err);
                    rimraf(zipPath);
                    yield fs_1.promises.writeFile(versionPath, pkg.version);
                    resolve();
                }));
            });
        });
        req.on('error', err => {
            reject(err);
        });
    }));
};
exports.createDistWindows = (baseDir, dest) => __awaiter(this, void 0, void 0, function* () {
    try {
        yield rimraf(dest);
        yield mkdirp(dest);
        const electronVersion = yield exports.getElectronVersion(baseDir);
        yield fs_1.promises.writeFile(path_1.join(dest, 'electron_version'), electronVersion.major);
        yield copy(path_1.join(__dirname, '../download/win32/electron.exe'), path_1.join(dest, 'electron.exe'));
    }
    catch (e) {
        console.error(e);
    }
});
exports.createDistLinux = (baseDir, dest) => __awaiter(this, void 0, void 0, function* () {
    try {
        yield rimraf(dest);
        yield mkdirp(dest);
        const electronVersion = yield exports.getElectronVersion(baseDir);
        yield fs_1.promises.writeFile(path_1.join(dest, 'electron_version'), electronVersion.major);
        yield copy(path_1.join(__dirname, '../download/linux/electron'), path_1.join(dest, 'electron'));
    }
    catch (e) {
        console.error(e);
    }
});
exports.createDistMac = (baseDir, dest) => __awaiter(this, void 0, void 0, function* () {
    try {
        const contentsPath = path_1.join(dest, 'Electron.app/Contents');
        const helperContentsPath = path_1.join(contentsPath, 'Frameworks/Electron Helper.app/Contents');
        yield rimraf(dest);
        yield mkdirp(dest);
        yield mkdirp(path_1.join(contentsPath, 'MacOS'));
        yield mkdirp(path_1.join(helperContentsPath, 'MacOS'));
        yield mkdirp(path_1.join(contentsPath, 'Resources'));
        const electronVersion = yield exports.getElectronVersion(baseDir);
        yield fs_1.promises.writeFile(path_1.join(contentsPath, 'Resources/electron_version'), electronVersion.major);
        yield fs_1.promises.writeFile(path_1.join(helperContentsPath, 'MacOS/Electron Helper'), '');
        yield Promise.all([
            copy(path_1.join(__dirname, '../download/darwin/electron'), path_1.join(contentsPath, 'MacOS/Electron')),
            copy(path_1.join(__dirname, '../resources/darwin/electron.plist'), path_1.join(contentsPath, 'Info.plist')),
            copy(path_1.join(__dirname, '../resources/darwin/electron-helper.plist'), path_1.join(helperContentsPath, 'Info.plist')),
        ]);
    }
    catch (e) {
        console.log(e);
    }
});
//# sourceMappingURL=index.js.map